/*
 ChessCheats, a chess program for the Apple iPhone.
 Copyright (C) 2011 Shawn E. Gano
 
 ChessCheats is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 ChessCheats is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//
//  ChessScreenshotModel.h
//  ChessCheats
//
//  Created by sgano on 4/11/11.
//  Copyright 2011 Shawn E. Gano. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MBProgressHUD.h"

extern float const percentageDownStartOfBoard;
extern float const sizeOfSquare;
//
extern float const whosTurnPercentDown;
extern float const whosTurnPercentRight;
extern float const whosTurnWidth;
extern float const whosTurnHeight;
//
extern float const playerColorPercentDown;
extern float const playerColorPercentRight;
extern float const playerColorWidth;
extern float const playerColorHeight;


@interface ChessScreenshotModel : NSObject 
{
    UIImage *screenshotImage; // the actual screenshot image
    bool imageFromIPad; // bool to tell if image was originally from iPad
    bool imageFromIPad2x;
    bool imageFromIPhoneRetina; // if originally from iPhone retina display
    bool playerWhite; // bool if player is white (or black) 
    NSMutableString *fen; // extracted FEN from the image -- is @"" if image hasn't been processed yet
    
    // http://stackoverflow.com/questions/2832729/how-to-convert-ascii-value-to-a-character-in-objective-c
    int board[8][8]; // holds chars for the board, array of arrays, containing NSStrings -- in the screen coordinate system
    float board_error[8][8]; // holds the ID errors
    
    // is it currently the players turn?
    bool isPlayersTurnAtBeginning; // YES if it is their turn at the begining, NO if not
    bool hasImageBeenProcessed;
    
    int enPassantScreenXCoord; // en passant square in screen shot coordinates (0-7)  <0 if none
    int enPassantScreenYCoord;
    
    //SEG v1.1 -- estimate of player color, is there a warning for this prediction?
    BOOL playerColorEstimateWarning;
}

//@property (nonatomic,retain) UIImage *screenshotImage; // nonatomic - so I can have a custom set method
@property (readonly) bool imageFromIPad;
@property (readonly) bool imageFromIPhoneRetina;
@property (nonatomic,readwrite) bool playerWhite;
@property (readwrite) bool hasImageBeenProcessed;
@property (readwrite) bool isPlayersTurnAtBeginning;
@property (readwrite) BOOL playerColorEstimateWarning; // SEG v1.1

-(BOOL) setScreenshotImage:(UIImage *)screenshot; // returns NO if there is an error
-(UIImage *)getScreenShotImage;
-(NSString *)getFEN;

-(bool) isSquareWhite:(int)i j:(int)j;
-(NSArray *)identifyChessPieceFromImage:(UIImage *)image isSquareWhite:(BOOL)isWhiteSquare;
-(BOOL)identifyWhosTurnFromImage:(UIImage *)image;
-(NSString *)generateShortFENwithHUD:(MBProgressHUD *)hud; // generates short fen and an estimate of Player's Color
-(void)flipFENandPlayerColor;

-(UIImage *)getScreenShotImage;

-(NSString *)peiceOnBoard_file:(int) i rank:(int)j;
-(float)errorOnBoard_file:(int)i rank:(int)j;
-(void)printBoard;
-(void)printErrorBoard;

-(int)possibleIdErrorCount;
-(NSString *)quickBoardValidityCheck; // returns error strings if there is a problem

-(void)updateFenStringFromBoardArray; //updates the fen string from the current board array
-(void)updateXPosition:(int)xBoardScreenPosition yPosition:(int)yBoardScreenPosition piece:(NSString *)piece;

-(int)getEnPassantScreenXCoord;
-(int)getEnPassantScreenYCoord;


+(NSArray *)estimateIfPlayerIsWhite:(int[8][8])boardPattern;
+ (NSArray*)getRGBAsFromImage:(UIImage*)image atX:(int)xx andY:(int)yy count:(int)count;
+ (NSArray *)whiteBlackPercentageFromImage:(UIImage*)image;
+ (NSArray *)whiteBlackPercentageFromImage:(UIImage*)image retinaSquare:(BOOL)iPhoneRetinaSquare iPad:(BOOL)iPad iPad2x:(BOOL)iPad2x;
+ (NSString *)determineCastleStringFromShortFEN:(NSString *)shortFEN;
+ (bool) checkKingOrginalPosition:(NSString *) rank isWhiteSide:(bool) whiteSide;
+ (UIImage *)imageWithImage:(UIImage *)image scaledToSize:(CGSize)newSize ;



@end
