/*
 ChessCheats, a chess program for the Apple iPhone.
 Copyright (C) 2011 Shawn E. Gano
 
 ChessCheats is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 ChessCheats is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//
//  ChessScreenshotModel.m
//  ChessCheats
//
//  Created by sgano on 4/11/11.
//  Copyright 2011 Shawn E. Gano. All rights reserved.
//

#import "ChessScreenshotModel.h"


@implementation ChessScreenshotModel

@synthesize imageFromIPad, imageFromIPhoneRetina, playerWhite, hasImageBeenProcessed,isPlayersTurnAtBeginning;
@synthesize playerColorEstimateWarning; // SEG v1.1


// Constants
float const percentageDownStartOfBoard = 91.0f/480.0f; // how far down from the top the image starts
float const sizeOfSquare = 40.0f; // size of the squares
// constants for who's turn
float const whosTurnPercentDown = 32.0f/480.0f;
float const whosTurnPercentRight = 103.0f/320.0f;
float const whosTurnWidth = 53.0f/320.0f;
float const whosTurnHeight = 20.0f/480.0f;
//
float const playerColorPercentDown = 441.0f/480.0f;
float const playerColorPercentRight = 165.0f/320.0f;
float const playerColorWidth = 148.0f/320.0f; // 88
float const playerColorHeight = 31.0f/480.0f;


// default init method
- (id)initWithImage:(UIImage *)screenshot
{
    self = [super init];
    if (self) 
    {
        fen = [NSMutableString stringWithString:@""];
        [fen retain];
        
        [self setScreenshotImage:screenshot];
        
        enPassantScreenXCoord = -1;
        enPassantScreenYCoord = -1;
        
        hasImageBeenProcessed = NO;
    }
    return self;
}

// without passing in a screenshot
-(id)init
{
    self = [super init];
    if (self) 
    {
        fen = [NSMutableString stringWithString:@""];
        [fen retain];
        
        //[self setScreenshotImage:screenshot];
        
        enPassantScreenXCoord = -1;
        enPassantScreenYCoord = -1;
        
        hasImageBeenProcessed = NO;
    }
    return self;
}

// return long FEN
-(NSString *)getFEN
{
    
    
     // Add whos turn , castleing, En Passant (not possible) and move numbers
     // http://en.wikipedia.org/wiki/Forsyth%E2%80%93Edwards_Notation
     // rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1
     
     NSString *whosTurnColor = playerWhite ? @"w" : @"b" ; // assumes it is "Your Turn"
    if(!isPlayersTurnAtBeginning)
    {
        whosTurnColor = playerWhite ? @"b" : @"w" ; // backwards as it is the computer's turn next
    }
     
    
     NSString *castleString =  [ChessScreenshotModel determineCastleStringFromShortFEN:fen];
    
    // en passant string
    NSString *ep = @"-";
    if([self getEnPassantScreenXCoord] >=0 && [self getEnPassantScreenYCoord] >= 0)
    {
        
        int x = [self getEnPassantScreenXCoord];
        int y = [self getEnPassantScreenYCoord];
        
        // okay there is an EP square -- lets figure out if it needs to be flipped
        // if player is white (not black) then we need to flip the position (since screen coords are upside down to start too)
        if(playerWhite)
        {
            y = 7-y;
        }
        else
        {
            x = 7-x;
        }
        
        switch (x) 
        {
            case 0:
                ep = [NSString stringWithFormat:@"a%i",y+1];
                break;
            case 1:
                ep = [NSString stringWithFormat:@"b%i",y+1];
                break;
            case 2:
                ep = [NSString stringWithFormat:@"c%i",y+1];
                break;
            case 3:
                ep = [NSString stringWithFormat:@"d%i",y+1];
                break;
            case 4:
                ep = [NSString stringWithFormat:@"e%i",y+1];
                break;
            case 5:
                ep = [NSString stringWithFormat:@"f%i",y+1];
                break;
            case 6:
                ep = [NSString stringWithFormat:@"g%i",y+1];
                break;
            case 7:
                ep = [NSString stringWithFormat:@"h%i",y+1];
                break;
                
            default:
                break;
        } // switch
        
    } // ep
    
     // put together the final FEN
     NSString *longFEN = [NSString stringWithFormat:@"%@ %@ %@ %@ 0 1",fen,whosTurnColor, castleString, ep];
     
     //DBOut(@"final fen: %@",fen);
        
    
    return longFEN;
}

-(BOOL) setScreenshotImage:(UIImage *)screenshot // returns NO - if there is an error
{
    // - do the processing --- set ipad/retina settings
    
    // quick checks -- is image too big to be a screenshot?
    if(screenshot.size.width >  1050) // not iPad or iPhone screenshot
    {
        return NO;
    }
    
    if(screenshotImage != nil)
    {
        // release the last one
        [screenshotImage release];
    }
    
    
    // reset defaults
    imageFromIPad = NO;
    imageFromIPhoneRetina = NO;
    imageFromIPad2x = NO;
    
    // SEG v1.1 -- clear player color warnings
    playerColorEstimateWarning = NO;
    
    
    
    
    [fen setString:@""]; // flags that FEN hasn't been solved for yet
    
    
    // reduce the image to 320/480 size -- smaller and faster to deal with
    
    // - retain the image? 
    
    ///////
    
    
        // -- do some pre-processing 
        float imageH = screenshot.size.height;
        float imageW = screenshot.size.width;
        float ar = imageH/imageW;
        
        if(ar < 1.4) // iPad 1024 x 768, ar = 1.333333
        {
            // now determine if 1x or 2x mode
            
            DBOut(@"iPad image");
            imageFromIPad = YES;
            imageFromIPhoneRetina = NO;
            
            float screenShotScale = 1.0f; // until an iPad has more than one resolution
            
            // screen shot is going to be centered and either 320 wide or 640 wide  [ipad width = 768]
            
            float x0 = (screenshot.size.width - 640.0f)/2.0f;
            float y0 = (screenshot.size.height- 960.0f)/2.0f + percentageDownStartOfBoard*960.0f;
            
            int i = 0;
            int j = 0;
            
            CGRect rr =  CGRectMake(x0+i*sizeOfSquare*screenShotScale, y0+j*sizeOfSquare*screenShotScale, sizeOfSquare*screenShotScale, sizeOfSquare*screenShotScale);
            CGImageRef square = CGImageCreateWithImageInRect(screenshot.CGImage, rr);
            UIImage *squareImage = [UIImage imageWithCGImage:square]; // contains just the square
            CGImageRelease(square);
            
            // get black percentage:
            float blackPerct = [(NSNumber *)[(NSArray *)[ChessScreenshotModel whiteBlackPercentageFromImage:squareImage] objectAtIndex:1] floatValue];
            
            //DBOut(@"black pect: %f",blackPerct);
            
            // use mode to just clip the image to just the needed game area
            if(blackPerct > 0.9) // x1 mode
            {
                DBOut(@"iPad image - 1x");
                
                float finalImageW = 320.0f;
                float finalImageH = 480.0f;
                
                
                CGRect r =  CGRectMake((screenshot.size.width - finalImageW)/2.0f,(screenshot.size.height- finalImageH)/2.0f,finalImageW,finalImageH);
                CGImageRef s = CGImageCreateWithImageInRect(screenshot.CGImage, r);
                UIImage *sImage = [UIImage imageWithCGImage:s]; // contains just the square
                CGImageRelease(s);
                
                //screenshotImage =  sImage;
                // this makes the processing MUCH MUCH FASTER!
                screenshotImage =  [ChessScreenshotModel imageWithImage:sImage scaledToSize:CGSizeMake(320, 480)];
                
                
            }
            else // x2 mode -- hopefully this won't exist soon!!
            {
                DBOut(@"iPad image - 2x");
                
                imageFromIPad2x = YES;
                
                float finalImageW = 320.0f*2.0f;
                float finalImageH = 480.0f*2.0f;
                
                
                CGRect r =  CGRectMake((screenshot.size.width - finalImageW)/2.0f,(screenshot.size.height- finalImageH)/2.0f,finalImageW,finalImageH);
                CGImageRef s = CGImageCreateWithImageInRect(screenshot.CGImage, r);
                UIImage *sImage = [UIImage imageWithCGImage:s]; // contains just the square
                CGImageRelease(s);
                
                // sImage now is twice as big as desired so shrink it before storing
                
                //screenshotImage =  [ChessScreenshotModel imageWithImage:sImage scaledToSize:CGSizeMake(320, 480)];
                // make IDing more consistant don't resize
                screenshotImage =  [ChessScreenshotModel imageWithImage:sImage scaledToSize:CGSizeMake(640, 960)];
                
                
                // test resize image back to smaller size  -- faster execution too
                // 
                //return [ProcessedImageView imageWithImage:sImage scaledToSize:CGSizeMake(320, 480)];
                
            }
        }
        else //iphone/ipod touch 480x320 (or 2x) ar = 1.5
        {
            // iphone/ipod touch
            imageFromIPad = NO; 
            
            // testing for retina:
            if(screenshot.size.width > 330)
            {
                imageFromIPhoneRetina = YES;
                
                
                
                // need to resize the image now --- THIS MAKES THE PROCESSING REALLY FAST???  MAY WANT TO LOOK INTO THIS???
                //screenshotImage = [ChessScreenshotModel imageWithImage:screenshot scaledToSize:CGSizeMake(320, 480)];
                // hmm try leaving in full res instead:
                //screenshotImage = screenshot;
                // Is this faster for processing later? -- scale to the exact same size -- this function must store the image in a way that is faster to process?  - does it take more memory?
                screenshotImage = [ChessScreenshotModel imageWithImage:screenshot scaledToSize:CGSizeMake(640, 960)];
                
            }
            else
            {
                imageFromIPhoneRetina = NO;
                
                // standard way
                //screenshotImage = screenshot;
                // Is this faster for processing later? -- scale to the exact same size -- this function must store the image in a way that is faster to process?  - does it take more memory?
                screenshotImage = [ChessScreenshotModel imageWithImage:screenshot scaledToSize:CGSizeMake(320, 480)];
            }
            
        }// iphone
    
    // retain the image
    [screenshotImage retain];
    
    // TO DO?  Clear BOARD?
        
    
    return YES;
    
} // setScreenshotImage

// V1.0.1 -- fixes bug when player color is changed, the fen is upside down -- because when the color is changed the FEN must be flipped
-(void)setPlayerWhite:(bool)isPlayerWhite
{
    playerWhite = isPlayerWhite;
    
    [self updateFenStringFromBoardArray]; // V1.0.1 -- this is the fix!
    
    // SEG v1.1 -- since user has specified the value, we need to now ignore any warnings
    playerColorEstimateWarning = NO;
}

-(UIImage *)getScreenShotImage
{
    return screenshotImage;
}

/*
// should return a copy maybe??
-(UIImage *)getScreenshotImage
{
    return screenshotImage;
}
 */


// finds all squares that might be wrong!!  
-(int)possibleIdErrorCount
{
    int errors = 0; 
    
    // Also use this value in ScreenshotOverlay.mm
    for(int i=0;i<8;i++)
    {
        for(int j=0;j<8;j++)
        {
            if([self errorOnBoard_file:i rank:j] > 1.0) // DATA NEED TO FILL IN (replace 1.0)
            {
                errors++;
            }
        } // j
    } // i
    
    // SEG v1.1 -- if player color estimate warning has flagged add 1
    if(playerColorEstimateWarning)
    {
        errors++;
    }
    
    return errors;
}


-(NSString *)quickBoardValidityCheck
{
    // if not processed just return "Image Not Processed"
    if(!hasImageBeenProcessed)
    {
        return @"Image Not Processed Yet";
    }
    
    NSMutableString * errorStr = [NSMutableString stringWithString:@""];
    
    // dictionary to keep track
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:12]; 
    
    // initialize dict
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"p"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"r"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"b"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"n"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"k"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"q"];
    
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"P"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"R"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"B"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"N"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"K"];
    [dict setObject:[NSNumber numberWithInt:0] forKey:@"Q"];
    
    // loop through board 
    for(int i=0;i<8;i++)
    {
        for(int j=0;j<8;j++)
        {
            int p = board[j][i];
            if(p != 0)
            {
                NSString * key = [NSString stringWithFormat:@"%c",p];
                NSNumber *n = (NSNumber *)[dict objectForKey:key];
                [dict setObject:[NSNumber numberWithInt:[n intValue]+1] forKey:key]; // ++
            }
            //    printf("   ");
            //else
            //    printf(" %c ",p);
        }
        //printf("\n");
    }
    
    // now loop through dict to see if there are any errors
    NSEnumerator *enumerator = [dict keyEnumerator];
    NSString * key;
    while ((key = (NSString *)[enumerator nextObject])) 
    {
       int count = [(NSNumber *)[dict objectForKey:key] intValue];
        
        //DBOut(@"%@ = %i", key, count);
        
        
        // knght, bishop, rook checks
        if([key caseInsensitiveCompare:@"N"] == NSOrderedSame || [key caseInsensitiveCompare:@"B"] == NSOrderedSame || [key caseInsensitiveCompare:@"R"] == NSOrderedSame)
        {
            if(count > 2)
            {
                [errorStr appendFormat:@"Too many: %@\n",key];
                //DBOut(@"here0");
            }
        }
        
        // pawns
        else if([key caseInsensitiveCompare:@"p"] == NSOrderedSame)
        {
            if(count > 8)
            {
                [errorStr appendFormat:@"Too many: %@\n",key];
                //DBOut(@"here1");
            }
        }
        
        // kings
        else if([key caseInsensitiveCompare:@"k"] == NSOrderedSame)
        {
            if(count > 1)
            {
                [errorStr appendFormat:@"Too many: %@\n",key];
               // DBOut(@"here2");
            }
            else if(count == 0)
            {
                [errorStr appendFormat:@"Too few: %@\n",key];
            }
        }
        
        // queens -- can have 0 or up to 9 -- is it really worth checking?
        else if([key caseInsensitiveCompare:@"q"] == NSOrderedSame)
        {
            if(count > 9)
            {
                [errorStr appendFormat:@"Too many: %@\n",key];
                //DBOut(@"here3");
            }
        }
        
    } // loop through dict
    
    
    return errorStr;
    
}// quickBoardValidityCheck


/////////
// generates FEN based on board
//  image just needs to only include the screenshot of the actual game (more than just the board) but not extra black if iPad
// http://en.wikipedia.org/wiki/Forsyth%E2%80%93Edwards_Notation
//
-(NSString *)generateShortFENwithHUD:(MBProgressHUD *)hud  // _isplayerWhite:(bool)isPlayerWhite  -- auto guesses if player is white or black
{
    //fen = [NSMutableString stringWithString:@""];
    [fen setString:@""];
    
    float imageH = screenshotImage.size.height;
    float imageW = screenshotImage.size.width;
    
       
    float screenShotScale = screenshotImage.size.width/320.0f;// in case of retina screenshot
    
    /////////////////////////////////////////////////
    // loop through all the squares on the board
    int blankCount = 0; // keep track of blanks in a row
    for(int j=0;j<8;j++)
    {
        for(int i=0;i<8;i++)
        {
            // ID each square's contents
            
            float dy = 0;
            if(imageFromIPhoneRetina) // need to make an adjustment for the retina display-- as the white border show up too much, skews results, makes empty spots on top row non-zero 
            {
                dy = 2.0;
            }
            
            CGRect rr =  CGRectMake(i*sizeOfSquare*screenShotScale, 0.0+percentageDownStartOfBoard*imageH+j*sizeOfSquare*screenShotScale+dy, sizeOfSquare*screenShotScale, sizeOfSquare*screenShotScale);
            CGImageRef square = CGImageCreateWithImageInRect(screenshotImage.CGImage, rr);
            UIImage *squareImage = [UIImage imageWithCGImage:square]; // contains just the square
            CGImageRelease(square);
            
            NSArray *pieceAndError = [self identifyChessPieceFromImage:squareImage isSquareWhite:[self isSquareWhite:i j:j]];
            NSString *piece = [pieceAndError objectAtIndex:0]; // piec string
            float error = [(NSNumber *)[pieceAndError objectAtIndex:1] floatValue];
            
            
            // if blank
            if([piece length] == 0)
            {
                blankCount++;
                
                board[i][j] = 0; // flag as nothing (null)
                board_error[i][j] = 0;
                
            }
            else  // we have a piece
            {
                if(blankCount > 0)
                {
                    // output blank count
                    [fen appendFormat:@"%i",blankCount];
                    blankCount = 0;
                }
                
                // add peice
                [fen appendString:piece];
                
                board[i][j] = [piece characterAtIndex:0]; // asciiCode
                board_error[i][j] = error; //
                
            } //we have a piece
            
            // HUD
            if(hud != nil)
            {
                hud.progress = (j*8+i)/(8.00f*8.0f); // 0.0f to 1.0f
            }
            
            
        } // i accross the board
        
        if(blankCount > 0)
        {
            [fen appendFormat:@"%i",blankCount];
        }
        
        if(j!=7) // not after the last line
        {
            [fen appendString:@"/"]; // next line
        }
        
        blankCount = 0; // reset
        
    } // j down the board
    
    
    // SEG v1.1 -- need to determine if there was a warning on estimating the color!
    NSArray * tempResults = [ChessScreenshotModel estimateIfPlayerIsWhite:board];
    playerWhite = [(NSNumber *)[tempResults objectAtIndex:0] boolValue];
    playerColorEstimateWarning = [(NSNumber *)[tempResults objectAtIndex:1] boolValue];
    ///
    
    /////////////////////////
    // determine if who's turn it is ----  isCurrentlyPlayersTurn, 33 down, 103 over, 47 wide /////////
    CGRect turn = CGRectMake(0+whosTurnPercentRight*1.0*imageW, 0+whosTurnPercentDown*imageH*1.0, 
                             whosTurnWidth*1.0*imageW, 0+(whosTurnHeight)*imageH*1.0);
    CGImageRef square = CGImageCreateWithImageInRect(screenshotImage.CGImage, turn);
    UIImage *squareImage = [UIImage imageWithCGImage:square]; // contains just the square
    CGImageRelease(square);
    isPlayersTurnAtBeginning = [self identifyWhosTurnFromImage:squareImage];
     /////////////////////////
    
    
    // if player is not white - we have to invert the board
    if(!playerWhite)
    {
        // flip FEN
        NSString * temp = [NSString stringWithString:fen];
        [fen setString:@""]; // clear string
        for(int i=[temp length]-1;i>=0;i--)
        {
            [fen appendFormat:@"%c",[temp characterAtIndex:i]];
        }
    }
    
    // tag as processed
    hasImageBeenProcessed = YES;
    
    // clearn En Passant Square
    enPassantScreenXCoord = -1;
    enPassantScreenYCoord = -1;
    
    
    return fen;
} // fen generator





// identify who's turn it is from the first word at the top -- their / your  return true if "your" false if their
-(BOOL)identifyWhosTurnFromImage:(UIImage *)image
{
    BOOL yourTurn = YES; // default guess
    
    NSArray *numArray = [ChessScreenshotModel whiteBlackPercentageFromImage:image];
    float blackPercentage = [(NSNumber *)[numArray objectAtIndex:1] floatValue];
    float whitePercentage = [(NSNumber *)[numArray objectAtIndex:0] floatValue];
    
    NSLog(@"whos turn  black, white:  %f, %f", blackPercentage, whitePercentage);
    
    ///////////////////////////////////////////
    ///////////////////////////////////////////
    //
    //  THIS DATA NEEDS TO BE INPUT BEFORE USE
    //
    ///////////////////////////////////////////
    ///////////////////////////////////////////
    
    float theirWhitePercentage = 0;
    float yourWhitePercentage = 0;
    
    if(imageFromIPad == NO && imageFromIPhoneRetina == NO) // regular iPhone
    {
        theirWhitePercentage = 0;
        yourWhitePercentage = 0;
    }
    else if(imageFromIPad == NO && imageFromIPhoneRetina == YES) // retina
    {
        theirWhitePercentage = 0;
        yourWhitePercentage = 0;
    }
    else if(imageFromIPad == YES && imageFromIPad2x == NO) // iPad 1x
    {
        theirWhitePercentage = 0;
        yourWhitePercentage = 0;
    }
    else if(imageFromIPad == YES && imageFromIPad2x == YES) // iPad 2x
    {
        theirWhitePercentage = 0;
        yourWhitePercentage = 0;
    }
    
    // see which error is less... that is our guess...
    DBOut(@"Turn ID:  white = %f", whitePercentage);
    
    if(  fabsf(whitePercentage - theirWhitePercentage) > fabsf(whitePercentage - yourWhitePercentage) )
    {
        // your turn
        DBOut(@"--- YOUR TURN ----");
        return YES;
    }
    else
    {
        // their turn
        DBOut(@"--- THEIR TURN ----");
        return NO;
    }

    
    
    return yourTurn;
}

// update the piece in the board at x,y location with the new piece -- used for editing when the auto detection messed up
-(void)updateXPosition:(int)xBoardScreenPosition yPosition:(int)yBoardScreenPosition piece:(NSString *)piece
{
    
    if([piece length] < 1) // blank
    {
        board[xBoardScreenPosition][yBoardScreenPosition] = 0; // flag as nothing (null)
        board_error[xBoardScreenPosition][yBoardScreenPosition] = -1; // since we are updating it we assume it is 100% accurate, flag it with -1 so in case we want to know which ones were set manually
        
        // clear EP check
        if(enPassantScreenXCoord == xBoardScreenPosition && enPassantScreenYCoord == yBoardScreenPosition)
        {
            enPassantScreenXCoord = -1;
            enPassantScreenYCoord = -1;
        }
        
    }
    else if([piece length] > 1) // en passant
    {
        enPassantScreenXCoord = xBoardScreenPosition;
        enPassantScreenYCoord = yBoardScreenPosition;
        
        // remove anything at that position if there was something
        board[xBoardScreenPosition][yBoardScreenPosition] = 0;
        board_error[xBoardScreenPosition][yBoardScreenPosition] = -1;//flag as marked
    }
    else // there is a piece 
    {
        board[xBoardScreenPosition][yBoardScreenPosition] = [piece characterAtIndex:0]; // asciiCode
        board_error[xBoardScreenPosition][yBoardScreenPosition] = -1; // since we are updating it assume correct and flag
        
        // clear EP check
        if(enPassantScreenXCoord == xBoardScreenPosition && enPassantScreenYCoord == yBoardScreenPosition)
        {
            enPassantScreenXCoord = -1;
            enPassantScreenYCoord = -1;
        }
    }
    
    // update short FEN string
    [self updateFenStringFromBoardArray]; 
}


-(void)updateFenStringFromBoardArray
{
   [fen setString:@""]; // clear string
    
    // update fen using screen coords
    /////////////////////////////////////////////////
    // loop through all the squares on the board
    int blankCount = 0; // keep track of blanks in a row
    for(int j=0;j<8;j++)
    {
        for(int i=0;i<8;i++)
        {
            // ID each square's contents
            
            
            // if blank
            if(board[i][j] == 0)
            {
                blankCount++;
            }
            else  // we have a piece
            {
                if(blankCount > 0)
                {
                    // output blank count
                    [fen appendFormat:@"%i",blankCount];
                    blankCount = 0;
                }
                
                NSString * piece = [NSString stringWithFormat:@"%c",board[i][j]];
                
                // add peice
                [fen appendString:piece];
                
            } //we have a piece
            
        } // i accross the board
        
        if(blankCount > 0)
        {
            [fen appendFormat:@"%i",blankCount];
        }
        
        if(j!=7) // not after the last line
        {
            [fen appendString:@"/"]; // next line
        }
        
        blankCount = 0; // reset
        
    } // j down the board
    
    
    
    // invert if needed
    // if player is not white - we have to invert the board
    if(!playerWhite)
    {
        // flip FEN
        NSString * temp = [NSString stringWithString:fen];
        [fen setString:@""]; // clear string
        for(int i=[temp length]-1;i>=0;i--)
        {
            [fen appendFormat:@"%c",[temp characterAtIndex:i]];
        }
    }
    
}//-(void)updateFenStringFromBoardArray;

-(int)getEnPassantScreenXCoord
{
    return enPassantScreenXCoord;
}

-(int)getEnPassantScreenYCoord
{
    return enPassantScreenYCoord;
}


-(NSString *)peiceOnBoard_file:(int) i rank:(int)j
{
    char p = (char)board[i][j];
    return [NSString stringWithFormat:@"%c",p];
}

-(float)errorOnBoard_file:(int)i rank:(int)j
{
    return board_error[i][j];
}

-(void)printBoard
{
    for(int i=0;i<8;i++)
    {
        for(int j=0;j<8;j++)
        {
            int p = board[j][i];
            if(p == 0)
                printf("   ");
            else
                printf(" %c ",p);
        }
        printf("\n");
    }
    
}

-(void)printErrorBoard
{
    for(int i=0;i<8;i++)
    {
        for(int j=0;j<8;j++)
        {
            int p = board[j][i];
            if(p == 0)
                printf(" (%f)  ",board_error[j][i]);
            else
                printf(" %c(%f) ",p,board_error[j][i]);
        }
        printf("\n");
    }
    
}


-(void)flipFENandPlayerColor
{
    
}

+(NSArray *)estimateIfPlayerIsWhite:(int[8][8])boardPattern
{
    bool estimateWhite = YES; // default guess
    
    // ASCII CODES
    // white
    int P = [@"P" characterAtIndex:0];
    int R = [@"R" characterAtIndex:0];
    int B = [@"B" characterAtIndex:0];
    int N = [@"N" characterAtIndex:0];
    int Q = [@"Q" characterAtIndex:0];
    int K = [@"K" characterAtIndex:0];
    // black
//    int p = [@"p" characterAtIndex:0];
//    int r = [@"r" characterAtIndex:0];
//    int b = [@"b" characterAtIndex:0];
//    int n = [@"n" characterAtIndex:0];
//    int q = [@"q" characterAtIndex:0];
//    int k = [@"k" characterAtIndex:0];
    
    // parameters used in estimate
    int whiteCount = 0; // just counting the lower 1/2 of the screenshot
    int blackCount = 0;
//    float whiteScore = 0;
//    float blackScore = 0;
    
    // start at j=4 -- just looking at bottom of the board in screenshot
    for(int j=4; j<8;j++)  // i/j in reference to screenshot  j=0 is top rank
    {
        for(int i=0; i<8; i++)
        {
            
            if(boardPattern[i][j] == 0)
            {
                // nothing this is blank spot
            }
            // else if white
            else if(boardPattern[i][j] == P || boardPattern[i][j] == R || boardPattern[i][j] == B || boardPattern[i][j] == N || boardPattern[i][j] == Q || boardPattern[i][j] == K)
            {
                whiteCount++;
            }
            // else black
            else
            {
                blackCount++;
            }
                
            
        } // i
    } // j
    
    if(blackCount > whiteCount)  // guess black only if they have a higher piece count in the bottom part of the screen
    {
        estimateWhite = NO; // looks like player is black
    }
    
    // SEG v1.1 -- add a measure for estimating when this guess might be bad
    int minDiffAllowed = 3;
    int minTotalCountForEstimatedColor = 3;
    BOOL warning = NO;// default no warning
    
    //DBOut(@"Back count = %i, White count = %i", blackCount, whiteCount); // SEG v1.1
    if( abs(blackCount - whiteCount) <  minDiffAllowed )
    {
        DBOut(@"-- PLAYER COLOR ESTIMATE WARNING: diff=%i --",abs(blackCount - whiteCount));
        warning = YES;
    }
    else if(estimateWhite && whiteCount < minTotalCountForEstimatedColor)
    {
        // estimated white, but total white count is low - so trigger warning
        DBOut(@"-- PLAYER COLOR ESTIMATE WARNING: low white = %i --",whiteCount);
        warning = YES;
    }
    else if(estimateWhite == NO && blackCount < minTotalCountForEstimatedColor)
    {
        // estimated black but total black count is low
        DBOut(@"-- PLAYER COLOR ESTIMATE WARNING: low black = %i --",blackCount);
        warning = YES;
    }
    //// end warning checks

    
    //return  estimateWhite;
    // SEG v1.1
    return [NSArray arrayWithObjects:[NSNumber numberWithBool:estimateWhite],[NSNumber numberWithBool:warning], nil];
    
} // estimateIfPlayerIsWhite
 


-(NSArray *)identifyChessPieceFromImage:(UIImage *)image isSquareWhite:(BOOL)isWhiteSquare
{
    
    NSArray *numArray = [ChessScreenshotModel whiteBlackPercentageFromImage:image retinaSquare:imageFromIPhoneRetina iPad:imageFromIPad iPad2x:imageFromIPad2x];
    float blackPercentage = [(NSNumber *)[numArray objectAtIndex:1] floatValue];
    float whitePercentage = [(NSNumber *)[numArray objectAtIndex:0] floatValue];
    
   

    
    
    // blank check
    if(blackPercentage < 0.001 && whitePercentage < 0.001)
    {
        return [NSArray arrayWithObjects:@"",[NSNumber numberWithFloat:0.0f], nil]; // blank square
    }
    
   
    ///////////////////////////////////////////
    ///////////////////////////////////////////
    //
    //  THIS DATA NEEDS TO BE INPUT BEFORE USE
    //
    ///////////////////////////////////////////
    ///////////////////////////////////////////
    
    //////
    // non-retina iphone (or iPad 1x ?)
    float onBlack[12][2];
    float onWhite[12][2];
    
    if(!imageFromIPhoneRetina && !imageFromIPad) // regular iPhone
    {
        float onBlackTemp[12][2] = {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };
        float onWhiteTemp[12][2] = {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };
        
        // copy over to offical array
        for(int i=0;i<12;i++)
        {
            onBlack[i][0] = onBlackTemp[i][0];
            onBlack[i][1] = onBlackTemp[i][1];
            
            onWhite[i][0] = onWhiteTemp[i][0];
            onWhite[i][1] = onWhiteTemp[i][1];
        }
        
    }
    else if(imageFromIPhoneRetina) // retina 
    {   //                                    P/p                 N                 R                  B                   Q                   K
        float onBlackTemp[12][2] = {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };
        
        
        float onWhiteTemp[12][2] = {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };
        
        
        // copy over to offical array
        for(int i=0;i<12;i++)
        {
            onBlack[i][0] = onBlackTemp[i][0];
            onBlack[i][1] = onBlackTemp[i][1];
            
            onWhite[i][0] = onWhiteTemp[i][0];
            onWhite[i][1] = onWhiteTemp[i][1];
        }

    }
    else if(imageFromIPad && imageFromIPad2x == NO) // iPad 1x
    {
        //                                    P/p                 N                 R                  B                   Q                   K
        float onBlackTemp[12][2] =   {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };     
        
        float onWhiteTemp[12][2] =   {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };
        
        // copy over to offical array
        for(int i=0;i<12;i++)
        {
            onBlack[i][0] = onBlackTemp[i][0];
            onBlack[i][1] = onBlackTemp[i][1];
            
            onWhite[i][0] = onWhiteTemp[i][0];
            onWhite[i][1] = onWhiteTemp[i][1];
        }
        
    }
    else if(imageFromIPad && imageFromIPad2x == YES) // iPad 2x
    {
        
        //                                    P/p                 N                 R                  B                   Q                   K
        float onBlackTemp[12][2] = {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };        
        
        float onWhiteTemp[12][2] = {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
            {0,0}, {0,0},{0,0},{0,0},{0,0},{0,0} };

        // copy over to offical array
        for(int i=0;i<12;i++)
        {
            onBlack[i][0] = onBlackTemp[i][0];
            onBlack[i][1] = onBlackTemp[i][1];
    
            onWhite[i][0] = onWhiteTemp[i][0];
            onWhite[i][1] = onWhiteTemp[i][1];
        }
         
    }
    //= {{},{},{},{},{},{},
    //    {}, {},{},{},{},{} };
    
    
    
    //////
    // for 2x mode -- though it should first be resized back to 1x - save time calculating the pixel counts
    //////
    
    // NSArray of strings ... P,N,....q,k
    NSArray * piecesStrings = [NSArray arrayWithObjects:@"P",@"N",@"R",@"B",@"Q",@"K",@"p",@"n",@"r",@"b",@"q",@"k", nil];
    
    int minPiece = 0;
    float minError = 0.0f;
    
    if(isWhiteSquare) // white square
    {
        minError = fabsf(blackPercentage - onWhite[0][0]) + fabsf(whitePercentage - onWhite[0][1]);
    }
    else // black square
    {
        minError = fabsf(blackPercentage - onBlack[0][0]) + fabsf(whitePercentage - onBlack[0][1]);
    }
    
    // loop through the rest
    for(int i=1;i<12;i++)
    {
        float error;
        if(isWhiteSquare) // white square
        {
            error = fabsf(blackPercentage - onWhite[i][0]) + fabsf(whitePercentage - onWhite[i][1]);
        }
        else // black square
        {
            error = fabsf(blackPercentage - onBlack[i][0]) + fabsf(whitePercentage - onBlack[i][1]);
        }
        
        if(error < minError)
        {
            minError = error;
            minPiece = i;
        }
        
    } // for 1..11
    
    NSArray * pieceAndErrorArray = [NSArray arrayWithObjects:[piecesStrings objectAtIndex:minPiece],[NSNumber numberWithFloat:minError], nil];
    
    return pieceAndErrorArray;//[piecesStrings objectAtIndex:minPiece];
    
} // identifyChessPieceFromImage


// resizing this to the same size seems to make the UIImage MUCH FASTER to analyze
// from: http://stackoverflow.com/questions/2658738/the-simplest-way-to-resize-an-uiimage
+ (UIImage *)imageWithImage:(UIImage *)image scaledToSize:(CGSize)newSize 
{
    UIGraphicsBeginImageContext(newSize);
    [image drawInRect:CGRectMake(0, 0, newSize.width, newSize.height)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();    
    UIGraphicsEndImageContext();
    return newImage;
}

+ (NSArray *)whiteBlackPercentageFromImage:(UIImage*)image
{
    return [ChessScreenshotModel whiteBlackPercentageFromImage:image retinaSquare:NO iPad:NO iPad2x:NO];
}


// white and black at the same time to speed things up
+ (NSArray *)whiteBlackPercentageFromImage:(UIImage*)image retinaSquare:(BOOL)iPhoneRetinaSquare iPad:(BOOL)iPad iPad2x:(BOOL)iPad2x
{
    //NSMutableArray *result = [NSMutableArray arrayWithCapacity:count];
    int whiteCount = 0;
    int blackCount = 0;
    
    // First get the image into your data buffer
    CGImageRef imageRef = [image CGImage];
    NSUInteger width = CGImageGetWidth(imageRef);
    NSUInteger height = CGImageGetHeight(imageRef);
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    unsigned char *rawData = malloc(height * width * 4);
    NSUInteger bytesPerPixel = 4;
    NSUInteger bytesPerRow = bytesPerPixel * width;
    NSUInteger bitsPerComponent = 8;
    CGContextRef context = CGBitmapContextCreate(rawData, width, height,
                                                 bitsPerComponent, bytesPerRow, colorSpace,
                                                 kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big);
    CGColorSpaceRelease(colorSpace);
    
    CGContextDrawImage(context, CGRectMake(0, 0, width, height), imageRef);
    CGContextRelease(context);
    
    // Now your rawData contains the image data in the RGBA8888 pixel format.
    int xx = 0; // start at the begining
    int yy = 0;
    int count = width * height; // all of them
    int byteIndex = (bytesPerRow * yy) + xx * bytesPerPixel;
    
    
    ///////////////////////////////////////////
    ///////////////////////////////////////////
    //
    //  THIS DATA NEEDS TO BE INPUT BEFORE USE
    //
    ///////////////////////////////////////////
    ///////////////////////////////////////////
    
    
    // filter constants -- defaults 
    double whiteFilter = 0;
    double blackFilter = 0; // 
    
    if(iPhoneRetinaSquare) // if retina iPhone
    {
        whiteFilter = 0;
        blackFilter = 0;
    }
    else if(iPad && iPad2x == YES) // iPad 2x
    {
        whiteFilter = 0; //94 is too low
        blackFilter = 0;
    }
    
    
    
    for (int ii = 0 ; ii < count ; ++ii)
    {
        CGFloat red   = (rawData[byteIndex]     * 1.0) / 255.0;
        CGFloat green = (rawData[byteIndex + 1] * 1.0) / 255.0;
        CGFloat blue  = (rawData[byteIndex + 2] * 1.0) / 255.0;
        //CGFloat alpha = (rawData[byteIndex + 3] * 1.0) / 255.0;
        byteIndex += 4;
        
        if(red > whiteFilter && green > whiteFilter && blue > whiteFilter) // close enough to white
        {
            whiteCount++;
        }
        
        if(red < (1.0-blackFilter) && green < (1.0-blackFilter) && blue < (1.0-blackFilter) ) // close enough to white
        {
            blackCount++;
        }
        
        //UIColor *acolor = [UIColor colorWithRed:red green:green blue:blue alpha:alpha];
        //[result addObject:acolor];
    }
    
    free(rawData);
    
    NSArray *array = [NSArray arrayWithObjects:[NSNumber numberWithFloat:(whiteCount*1.0f)/(width*height*1.0f)],
                      [NSNumber numberWithFloat:(blackCount*1.0f)/(width*height*1.0f)],
                      nil];
    
    return array;
    
} // whiteBlackPercentageFromImage


////
// http://stackoverflow.com/questions/448125/how-to-get-pixel-data-from-a-uiimage-cocoa-touch-or-cgimage-core-graphics
////
+ (NSArray*)getRGBAsFromImage:(UIImage*)image atX:(int)xx andY:(int)yy count:(int)count
{
    NSMutableArray *result = [NSMutableArray arrayWithCapacity:count];
    
    // First get the image into your data buffer
    CGImageRef imageRef = [image CGImage];
    NSUInteger width = CGImageGetWidth(imageRef);
    NSUInteger height = CGImageGetHeight(imageRef);
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    unsigned char *rawData = malloc(height * width * 4);
    NSUInteger bytesPerPixel = 4;
    NSUInteger bytesPerRow = bytesPerPixel * width;
    NSUInteger bitsPerComponent = 8;
    CGContextRef context = CGBitmapContextCreate(rawData, width, height,
                                                 bitsPerComponent, bytesPerRow, colorSpace,
                                                 kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big);
    CGColorSpaceRelease(colorSpace);
    
    CGContextDrawImage(context, CGRectMake(0, 0, width, height), imageRef);
    CGContextRelease(context);
    
    // Now your rawData contains the image data in the RGBA8888 pixel format.
    int byteIndex = (bytesPerRow * yy) + xx * bytesPerPixel;
    for (int ii = 0 ; ii < count ; ++ii)
    {
        CGFloat red   = (rawData[byteIndex]     * 1.0) / 255.0;
        CGFloat green = (rawData[byteIndex + 1] * 1.0) / 255.0;
        CGFloat blue  = (rawData[byteIndex + 2] * 1.0) / 255.0;
        CGFloat alpha = (rawData[byteIndex + 3] * 1.0) / 255.0;
        byteIndex += 4;
        
        UIColor *acolor = [UIColor colorWithRed:red green:green blue:blue alpha:alpha];
        [result addObject:acolor];
    }
    
    free(rawData);
    
    return result;
} // getRGBAsFromImage



// i is left to right 0-7 starting from the top, j = 0,7, starting from the top to bottom
-(bool)isSquareWhite:(int)i j:(int)j
{
    if( j % 2 == 0)
    {  // row is even
        if( i % 2 == 0) // column is even
        {
            return YES;
        }
        else // col is odd
        {
            return NO;
        }
    }
    else // row odd
    {
        if( i % 2 == 0) // column is even
        {
            return NO;
        }
        else // col is odd
        {
            return YES;
        }
    }
} // isSquareWhite


+(NSString *)determineCastleStringFromShortFEN:(NSString *)shortFEN
{
    NSMutableString *castle = [NSMutableString stringWithString:@""];
    
    NSArray *lines = [shortFEN componentsSeparatedByString:@"/"];
    
    if( [lines count] != 8 )
    {
        DBOut(@"WARNING: FEN DOESN'T HAVE 8 LINES (Castleing check)");
        return @"-"; // return nothing
    }
    
    
    // this is a check on the White side
    NSString *baseWhiteRank = (NSString *)[lines objectAtIndex:7];
    if([ChessScreenshotModel checkKingOrginalPosition:baseWhiteRank isWhiteSide:YES ])
    {
        if( [baseWhiteRank hasSuffix:@"R"])
        {
            [castle appendString: @"K"];
            //System.out.println("KING SIDE CASTLING (White) IS POSSIBLE!! --  " + line);
        }// King side
        
        if([baseWhiteRank hasPrefix:@"R"])
        {
            [castle appendString: @"Q"]; //castling += "Q";
            //System.out.println("QUEEN SIDE CASTLING (White) IS POSSIBLE!! --  " + line);
        }// Queen side
        
    } // king is in correct spot
    ///////     
    // this is a check on the Black side
    NSString *baseBlackRank = (NSString *)[lines objectAtIndex:0];
    if([ChessScreenshotModel checkKingOrginalPosition:baseBlackRank isWhiteSide:NO ])
    {
        if( [baseBlackRank hasSuffix:@"r"])//if(parts[0].endsWith("r"))
        {
            [castle appendString: @"k"];//castling += "k";
            //System.out.println("KING SIDE CASTLING (Black) IS POSSIBLE!! --  " + line);
        }// King side
        
        if( [baseBlackRank hasPrefix:@"r"])//if(parts[0].startsWith("r"))
        {
            [castle appendString: @"q"];//castling += "q";
            //System.out.println("QUEEN SIDE CASTLING (Black) IS POSSIBLE!! --  " + line);
        }// Queen side
        
    } // king is in correct spot
    ////////     
    // no castling moves found:
    if([castle length] == 0)
    {
        [castle setString:@"-"]; // return just a dash
    }
    
    
    
    return castle;
} // determineCastleStringFromShortFEN



// takes a row and finds if King has moved from orginal position returns true if it is
// boolean - true for the white side check, false for the back side check
+ (bool) checkKingOrginalPosition:(NSString *) rank isWhiteSide:(bool) whiteSide
{
    bool result = NO;
    NSString *kingStr = whiteSide ? @"K" : @"k";
    
    // first check to make sure there is a K in this row
    if([(NSArray *)[rank componentsSeparatedByString:kingStr] count] == 1)
    {
        return NO; // no king - no need to look any further
    }
    
    // okay find the position of the king
    int kingLoc = 0; // location of the king  1-8
    for(int i=0; i<[rank length]; i++)
    {
        char c = [rank characterAtIndex:i];
        NSString *cc = [NSString stringWithFormat:@"%c",c];
        
        // see if it is an int - if not just skip on location, otherwise skip int number of positions
        if([cc isEqualToString:kingStr])
        {
            // king found!
            kingLoc++;
            goto endsearch;
        }
        
        // see if next char is an int or a letter
        int s = [cc intValue];
        if(s == 0 ) // not an integer found
        {
            // okay not an int - some other piece - just skip one spot
            kingLoc++;
        }
        else // number was found - skip that number of places
        {
            kingLoc += s;
        }
        
    } // search for loop
endsearch:;
    
    
    if(kingLoc == 5)
    {
        result = YES;
    }
    
    return result;
} // checkKingOrginalPosition


- (void)dealloc
{
    [screenshotImage release];
    
    [super dealloc];
}


@end
