/*
 ChessCheats, a chess program for the Apple iPhone.
 Copyright (C) 2011 Shawn E. Gano
 
 ChessCheats is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 ChessCheats is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//
//  MainScreenViewController.m
//  ChessCheats
//
//  Created by sgano on 4/11/11.
//  Copyright 2011 Shawn E. Gano. All rights reserved.
//

#import "MainScreenViewController.h"
#import "Options.h"

@implementation MainScreenViewController

@synthesize boardViewController,overlayView,screenshotImageView;


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [chessModel release];
    
    // SEG blank board bug fix
    [savedRootView release];
    
    // SEG v1.3
    [backgroundImageView release];
    
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    
    // blank board bug fix
    if(savedRootView != nil)
    {
        [self setView: savedRootView];
        // DO IT FOR Ipad to.....
        
        // resetup schrenshotimage view
        //overlayView.screenshotImageView = screenshotImageView;
        //overlayView.chessModel = chessModel;
        
        DBOut(@"RETURNING SAVED VIEW: BoardViewController");
        return;
    }
    
    
    // set oFverlayView - fixes second part of memory warning bug 
    self.overlayView =  _overlayView;
    self.screenshotImageView = _screenshotImageView;
    
    
    DBOut(@"!!!!!!!!!!!!!!!VIEW DID LOAD!!!!!!!!!!!!!!!!!!!");
    self.title = @"Chess Cheats";
    
    ///////////// DEBUG ///////  LOAD INI FILE TO TEST /////////////
    //NSString *fileLocation = [[NSBundle mainBundle] pathForResource:@"chess_friends" ofType:@"PNG"];
    BOOL debug = NO;
    if(debug)
    {
        NSString *fileLocation = [[NSBundle mainBundle] pathForResource:@"chess_new" ofType:@"png"]; // chess_new  photo_end_game
        NSData *imageData = [NSData dataWithContentsOfFile:fileLocation];
        
        UIImage *processedImage  = [UIImage imageWithData:imageData];
        
        // setup the chessModel
        chessModel = [[ChessScreenshotModel alloc] initWithImage:processedImage];
        [chessModel retain];
        self.overlayView.chessModel = chessModel;
        
        //screenshotImage= [ProcessedImageView processSelectedImage:processedImage];
        //[screenshotImageView setImage:screenshotImage];
        [screenshotImageView setImage:[chessModel getScreenShotImage]]; // this has been processed
    }
    /////////////////////////
    
    // pass screenshotImageView to overlay view
    self.overlayView.screenshotImageView = screenshotImageView;
    
    
    
    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] 
                                  initWithImage:[UIImage imageNamed:@"18-envelope"]  
                                  style:UIBarButtonItemStyleBordered target:self 
                                  action:@selector(emailFEN)];
    self.navigationItem.rightBarButtonItem = addButton;
    
    // SEG V1.1 -- add express button
    // SEG V2.0 -- add social button
    UIBarButtonItem *socialButton = [[[UIBarButtonItem alloc]
                                      initWithImage:[UIImage imageNamed:@"63-runner"]
                                      style:UIBarButtonItemStyleBordered // UIBarButtonItemStyleBordered  UIBarButtonItemStylePlain
                                      target:self
                                      action:@selector(expressLoad)] autorelease];
	self.navigationItem.leftBarButtonItem = socialButton;
    
    //set variables to Processed Omage View
//    overlayView.screenshotImageView = screenshotImageView;
  //  overlayView.screenshotImage = screenshotImage;
  //  overlayView.whiteText = whiteText;
   // overlayView.blackText = blackText;
    
    //[overlayView retain];
    
    
    // blank board bug fix
    savedRootView = self.view;
    [savedRootView retain];
    
    
    
    // SEG v1.3 -- set background image for iPad
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
    {
        [backgroundImageView setImage:[UIImage imageNamed:@"background_ipad"]];
    }

    
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    processingExpressLoad = NO; // SEG v1.1 clear flag
    
    
    // Create an inital "help prompt" the first time someone starts the app
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    if (![defaults objectForKey: @"firstTimeStartMain"]) 
    {
        // Alert
        [[[[UIAlertView alloc] initWithTitle: @"Welcome!"
                                     message: @"A couple of tips to help you get started: \n\n1)  First create a screenshot from your chess game by pressing the power button and the home button at the same time.\n\n2)  Using this app load the screenshot by selecting the camera icon in the lower left."
                                    delegate: nil
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        [defaults setBool: YES forKey: @"firstTimeStartMain"];
        [defaults synchronize];
        
        [self edit]; // show help bubbles
    }
    
} // viewDidAppear

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

// SEG v1.1 -- alert view clicked button... to resend back to expressLoad
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    // express welcon screen hit
    if([alertView.title compare:@"Express Button!"] == NSOrderedSame)
    {
        // SEG v1.2 -- bug fix -- reset the express Load processing so it can run on this next call
        processingExpressLoad = NO;
        
        [self expressLoad]; // rerun -- this time popup won't interfear with the "Location request" dialog
    }
}

// SEG V 1.1  Express Load button
-(void)expressLoad
{
    // -- Alert user as to what this button does the first (and only the first) time they run it...
    DBOut(@"Express Load Started");
    
    if(processingExpressLoad)
    {
        DBOut(@"Double express button hit (1)");
        return;
    }
    
    processingExpressLoad = YES;
    
    // Create an inital "help prompt" the first time someone hits the express button
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    if (![defaults objectForKey: @"firstTimeExpressButton"]) 
    {
        // Alert
        [[[[UIAlertView alloc] initWithTitle: @"Express Button!"
                                     message: @"This button loads the last image in your camera roll (which should be your last screenshot) and if there are no errors the board will be automatically opened! It's fast!\n\nIf you get a 'Current Location' warning just hit OK - this is needed to access the image library (note: this app does NOT use any location information)."
                                    delegate: self
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        [defaults setBool: YES forKey: @"firstTimeExpressButton"];
        [defaults synchronize];
        
        return; // wait for user to click and catch in delagate
    }
    
    
    // load last image 
    // http://stackoverflow.com/questions/6336770/access-a-single-image-with-alassetslibrary
    // http://developer.apple.com/library/ios/#documentation/AssetsLibrary/Reference/ALAssetsLibrary_Class/Reference/Reference.html
    // and the WWDC 2010 session 421 video
    
    ALAssetsLibrary *assetsLibrary = [[ALAssetsLibrary alloc] init];
    
    
    ALAssetsGroupEnumerationResultsBlock groupResults =  ^(ALAsset *result, NSUInteger index, BOOL *stop)
    {
        if(!result)
        {
            // smething went wrong... alert user... return
            //DBOut(@"Error getting last image");
            //processingExpressLoad = NO;
            
            // this may normally get called nil when finished
            
            return;
        }
        
        //NSLog(@"Got last image");
        
        ALAssetRepresentation *rep = [result defaultRepresentation];
        
        
        
        UIImage * img = [UIImage imageWithCGImage:[rep fullResolutionImage]];
        
        // DEBUG... only works iPhone -- this is just a test
        [screenshotImageView setImage:img]; // set the image
        
        [self loadNewImage:img autoLoad:YES]; // load image and auto-load board
        
                
        *stop = YES; // all done
    };
    
    ALAssetsLibraryGroupsEnumerationResultsBlock listGroupBlock = ^(ALAssetsGroup *group, BOOL *stop)
    {
        //NSLog(@"Start enumeration block");
        
        if(group) // will only run once because we just asked for ALAssetsGroupSavedPhotos
        {
           // NSLog(@"Group Name: %@",[group valueForProperty:ALAssetsGroupPropertyName]);
            
            // 
            // index set for the very last index
            NSIndexSet *lastIndexSet = [NSIndexSet indexSetWithIndex:[group numberOfAssets]-1];
            
            //NSLog(@"num assets: %i",[group numberOfAssets]);
            
            [group enumerateAssetsAtIndexes:lastIndexSet options:nil usingBlock:groupResults];
            
        }
    };
    
    ALAssetsLibraryAccessFailureBlock failureBlock = ^(NSError *error)
    {
        // data not accessable error.... prob a user setting!!
        //NSLog(@"Access Error");
        // error found
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error Getting Image" 
                                                        message:[NSString stringWithFormat:@"Error getting image from the Camera Roll photo album. \n\nPlease insure that Settings -> Location Services -> ChessCheats is turned on."]
                                                       delegate:nil 
                                              cancelButtonTitle:@"OK" 
                                              otherButtonTitles: nil];
        [alert show];
        [alert release];
        
        // reset processing flag
        processingExpressLoad = NO;
    };
    
    // get group
    //As @mithuntnt mentioned, you can get the ALAssetsGroup for the photo library using:
    // ALAssetsGroupSavedPhotos =  Retrieve the images saved in the Camera role
    [assetsLibrary enumerateGroupsWithTypes:ALAssetsGroupSavedPhotos usingBlock:listGroupBlock failureBlock:failureBlock];
    
    
    
    
} // express load -- v1.1


-(void)emailFEN
{
    // TODO -- fill this out -- (make sure there is an image --> then make sure it is processed --> export FEN to e-mail)
    
    if(chessModel == nil)
    {
        [[[[UIAlertView alloc] initWithTitle: @"No Screenshot"
                                     message: @"You need to choose a screenshot before you can e-mail the chess game.  Select the camera icon below to get started."
                                    delegate: nil
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        return;
    }
    else if(chessModel.hasImageBeenProcessed == NO)
    {
        [[[[UIAlertView alloc] initWithTitle: @"Image hasn't been processed"
                                     message: @"You need to choose a screenshot and process it before you can e-mail the chess game.  Select the camera icon below to get started."
                                    delegate: nil
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        return;
    }

    
    
    if (![MFMailComposeViewController canSendMail]) // if device is not in a state that can send e-mail
    {
        DBOut(@"Device is unable to send email in its current state.");
        return;
    }
    
    // CREATING MAIL VIEW
    MFMailComposeViewController *controller = [[MFMailComposeViewController alloc] init];
    controller.mailComposeDelegate = self;
    [controller setSubject:@"Chess Game"];
    [controller setMessageBody:[NSString stringWithFormat:@"Game FEN: \n%@\n\n",[chessModel getFEN]] isHTML:NO];
    
    // MAKING A SCREENSHOT
    //UIGraphicsBeginImageContext(_mapView.frame.size);
    //[_mapView.layer renderInContext:UIGraphicsGetCurrentContext()];
    //UIImage *screenshot = UIGraphicsGetImageFromCurrentImageContext();
    //UIGraphicsEndImageContext();
    
    // ATTACHING A SCREENSHOT
    NSData *myData = UIImagePNGRepresentation([chessModel getScreenShotImage]);
    [controller addAttachmentData:myData mimeType:@"image/png" fileName:@"route"]; 
    
    // SHOWING MAIL VIEW
    [self presentModalViewController:controller animated:YES];
    [controller release];
    
}

-(void)mailComposeController:(MFMailComposeViewController*)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error 
{
        
    [self dismissModalViewControllerAnimated:YES];
}


// toolbox is pushed
-(IBAction)edit
{
    UIButton * helpOverlayButton = [[UIButton alloc] initWithFrame:self.view.frame];
    
    NSString * helpBubbleImage = @"main_page_help_overlay";
    
    // if iPad use different image
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
    {
        helpBubbleImage = @"main_page_help_overlay_ipad.png";
    }
    
    // set background image
    [helpOverlayButton setBackgroundImage:[UIImage imageNamed:helpBubbleImage] forState:UIControlStateNormal];
    [helpOverlayButton setBackgroundImage:[UIImage imageNamed:helpBubbleImage] forState:UIControlStateSelected];
    [helpOverlayButton setBackgroundImage:[UIImage imageNamed:helpBubbleImage] forState:UIControlStateHighlighted];
    
    // add selector
    [helpOverlayButton addTarget:self action:@selector(closeHelpOverlay:) forControlEvents:UIControlEventTouchUpInside];
    
    [helpOverlayButton setAlpha:0.0f]; // start invisiable
    
    // add button to the top
    [self.view addSubview:helpOverlayButton];
    [self.view bringSubviewToFront:helpOverlayButton];
    
    // animate fade in
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.5]; 
        [helpOverlayButton setAlpha:1.0f];
    [UIView commitAnimations];

    
}

-(void)closeHelpOverlay:(id)sender
{
    //[sender removeFromSuperview];
    
    // using blocks -- is clean and actually removes the subview properly
    [UIView animateWithDuration:0.5
                     animations:^{[sender setAlpha:0.0f];}
                     completion:^(BOOL finished){[sender removeFromSuperview];}];
    
}

// SEG v1.1 -- added as a passthrough to support IBAction
-(IBAction)processImage
{
    DBOut(@"PROCESS BUTTON HIT");
    
    // first check to make sure we aren't already loading an image from the express button
    if(processingExpressLoad)
    {
        DBOut(@"Double express button hit (2)");
        return;
    }
    
    
    [self processImage_autoOpenBoardNoErrors:[NSNumber numberWithBool:NO]]; // default is don't auto open
}

-(void)processImage_autoOpenBoardNoErrors:(NSNumber *)autoOpenNSNumber  // SEG v1.1 added autoOpen
{
    BOOL autoOpen = [autoOpenNSNumber boolValue];
    
    if(chessModel == nil)
    {
        [[[[UIAlertView alloc] initWithTitle: @"No Screenshot"
                                     message: @"You need to choose a screenshot before you can process it.  Select the camera icon on the left to get started."
                                    delegate: nil
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        // v1.1 reset flag
        processingExpressLoad = NO;
        
        return;
    }
    else if( [chessModel getScreenShotImage] == nil)
    {
        [[[[UIAlertView alloc] initWithTitle: @"No Screenshot"
                                     message: @"You need to choose a screenshot before you can process it.  Select the camera icon on the left to get started."
                                    delegate: nil
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        // v1.1 reset flag
        processingExpressLoad = NO;
        
        return;
    }
    
    // preliminaries -- close any popups that may be up
    [self.overlayView closePopups];
    
    
    // prepare HUD
    hud = [[MBProgressHUD alloc] initWithView:self.navigationController.view];
	[self.navigationController.view addSubview:hud];
	
    // Set determinate mode
    hud.mode = MBProgressHUDModeDeterminate;
    
	hud.delegate = self;
    hud.labelText = @"Processing Screenshot";
	
	// task uses the HUD instance to update progress
    [hud showWhileExecuting:@selector(processImageBackground_autoOpenBoardNoErrors:) onTarget:self withObject:[NSNumber numberWithBool:autoOpen] animated:YES];
    
}

// can be used to process image as a selector in the background
-(void)processImageBackground_autoOpenBoardNoErrors:(NSNumber *)autoOpenNSNumber // SEG v1.1 added autoOpen
{
    // v1.1 extract BOOL
    BOOL autoOpen = [autoOpenNSNumber boolValue];
    
    double beforeTime = CACurrentMediaTime();
    [chessModel generateShortFENwithHUD:hud];
    double afterTime = CACurrentMediaTime();
    
    DBOut(@"Short FEN: %@",[chessModel getFEN]);
    DBOut(@"Estimated player color: %@", chessModel.playerWhite ? @"white" : @"black");
    DBOut(@"Processing took: %f seconds", afterTime-beforeTime);
    
    [self.overlayView updateDisplay]; // update view
    
    //[chessModel printBoard];
    
    // TODO... maybe some simple checks... like are there 2 or less knights, etc.  to alert user if something is fishy.
    NSString * errorStr = [chessModel quickBoardValidityCheck];
    
    if([errorStr length] > 0)
    {
        // error found
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Errors Detected!" 
                                                        message:[NSString stringWithFormat:@"The following error(s) were detected please correct before continuing:\n\n%@",errorStr]
                                                       delegate:nil 
                                              cancelButtonTitle:@"OK" 
                                              otherButtonTitles: nil];
        [alert show];
        [alert release];
    }
    else if([chessModel possibleIdErrorCount] > 0) // SEG v1.1 -- this now includes a check for the Player color estimate
    {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Possible Errors Detected!" 
                                                        message:[NSString stringWithFormat:@"Check the pink highlighted squares to insure they have been identified correctly; if needed tap on the square to fix it."]
                                                       delegate:nil 
                                              cancelButtonTitle:@"OK" 
                                              otherButtonTitles: nil];
        [alert show];
        [alert release];
    }
    // v1.1 -- if no errors see if we are supposed to autoOpen board
    else if(autoOpen)
    {
        BOOL autoMakeFirstMove = [Options sharedOptions].autoMoveExpress; //  pull this from options
        NSNumber * autoMakeFirstMoveNSNumber = [NSNumber numberWithBool:autoMakeFirstMove];
        
        [self performSelectorOnMainThread:@selector(showChessBoard_autoFirstMove:) withObject:autoMakeFirstMoveNSNumber waitUntilDone:NO];
        //[self showChessBoard];
        //NSLog(@"should auto open board");
    }
    
    // v1.1 reset flag
    processingExpressLoad = NO;
    
}

///////  Chose image from Library ----------------
-(IBAction)chooseImage
{
    
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
    {
        // iPad must have image picker in a popup
        UIImagePickerController *imagePicker = [[UIImagePickerController alloc] init];
        imagePicker.delegate = self;
        
        popover = [[UIPopoverController alloc] initWithContentViewController:imagePicker] ;
        [imagePicker release];
        [popover presentPopoverFromRect:CGRectMake(30, 990.0, 0.0, 0.0) 
                                 inView:self.view
               permittedArrowDirections:UIPopoverArrowDirectionDown 
                               animated:YES];
        
        
    }
    else // iPhone
    {
        //DBOut(@"button hit");
        UIImagePickerController *picker = [[UIImagePickerController alloc] init];
        picker.delegate = self;
        
        picker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
        
        [self presentModalViewController:picker animated:YES];
        
        [picker release];
    }
    
}

-(void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
    {
        [popover dismissPopoverAnimated:YES];
        [popover release];
    }
    else
    {
        [picker dismissModalViewControllerAnimated:YES];
    }
    
    UIImage * selectedImage = [info objectForKey:@"UIImagePickerControllerOriginalImage"];
    
    if(selectedImage == nil)
    {
        return;
    }
    
    // seg V1.1 - moved out to a new function
    [self loadNewImage:selectedImage autoLoad:NO];
    
}


// SEG v1.1 - moved out of UIPicker responder, if returns NO then there was an error
-(BOOL)loadNewImage:(UIImage *)selectedImage autoLoad:(BOOL)autoload
{
    //// check if chessModel exists yet
    BOOL error = NO;
    if(chessModel == nil)
    {
        chessModel = [[ChessScreenshotModel alloc] init]; // initWithImage:selectedImage <- don't do this because we will do it below - to get error flag back
        [chessModel retain];
        
        [self.overlayView clearBoard];
        
        self.overlayView.chessModel = chessModel;
        self.overlayView.screenshotImageView = screenshotImageView;
    }
    else
    {
        // clear the peices and the last picked square
        [self.overlayView clearBoard];
    }
    
    // Do some inital processing on the image - take care of iPad/iPhone image extraction
    //UIImage *processedImage = [ProcessedImageView processSelectedImage:selectedImage];
    // update chess model
    error = [chessModel setScreenshotImage:selectedImage];  // should this clear the board???  
    
    
    // Error!
    if(error == NO)
    {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Image Error" 
                                                        message:[NSString stringWithFormat:@"This image does not appear to be an iPhone or iPad screenshot; please select another one.\n\n Hint: To create a screenshot push the power button and the home button at the same time."]
                                                       delegate:nil 
                                              cancelButtonTitle:@"OK" 
                                              otherButtonTitles: nil];
        [alert show];
        [alert release];
        
        // v1.1 -- reset processing flag (may not always be needed)
        processingExpressLoad = NO;
        
        
        return error;// get out of here
    }
    
    if(chessModel.imageFromIPad)
    {
        // use the model's image for the screen shot (clipped)
        [screenshotImageView setImage:[chessModel getScreenShotImage]];
    }
    else // just use the iPhone/iPod screenshot
    {
        [screenshotImageView setImage:selectedImage]; // set the image
    }
    
    // TO DO -- check for image not being a chess board??? before processing? (wrong aspect ratios - not ipad or iPhone) for instance.
    
    
    
    // process image after a slight delay to allow hiding animation to complete
    //[self performSelector:@selector(processImage) withObject:nil afterDelay:0.3 ];
    // V1.1 
    [self performSelector:@selector(processImage_autoOpenBoardNoErrors:) withObject:[NSNumber numberWithBool:autoload] afterDelay:0.3 ];

    
    //processedView.screenshotImage = screenshotImageView.image; // update image
    //[processedView setNeedsDisplay]; // update
    
    return YES; // returns successeded
}

/////////////////////////////

// SEG v1.2 -- takeing the place of the older function to allow passing of "auto move" option
-(IBAction)showChessBoard
{
    [self showChessBoard_autoFirstMove:[NSNumber numberWithBool:NO]];
}

// SEG v1.2 -- add autoMakeFirstMove option
-(void)showChessBoard_autoFirstMove:(NSNumber *)autoMakeFirstMoveNSNumber
{
    // v1.2 extract BOOL
    BOOL autoMove = [autoMakeFirstMoveNSNumber boolValue];
    
    ////////////////////////////////////////
    
    // have to save this because when we start the new game the level gets reset by internals -- so we will have to set this back
    // bogus
    //GameLevel gl_saved = [Options sharedOptions].gameLevel;
    
    if(chessModel == nil)
    {
        [[[[UIAlertView alloc] initWithTitle: @"No Screenshot"
                                     message: @"You need to choose a screenshot before you can open the chess game.  Select the camera icon on the left to get started."
                                    delegate: nil
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        return;
    }
    else if(chessModel.hasImageBeenProcessed == NO)
    {
        [[[[UIAlertView alloc] initWithTitle: @"Image hasn't been processed"
                                     message: @"You need to choose a screenshot and process it before you can open the chess game.  Select the camera icon on the left to get started."
                                    delegate: nil
                           cancelButtonTitle: nil
                           otherButtonTitles: @"OK", nil] autorelease]
         show];
        
        return;
    }
    
    
    // prep board
    [boardViewController initBoardForNewGame];
    
    
    [Options sharedOptions].gameMode = GAME_MODE_TWO_PLAYER; // default gameMode
    
    // game mode must be set before the new gameFromFEN is loaded
    if([chessModel playerWhite])//[(NSNumber *)[puzzle valueForKey:@"computerIsBlack"] boolValue])
	{
        // player is white
        if([Options sharedOptions].autoMoveOpponent) // if auto move is on change game mode
        {
            [Options sharedOptions].gameMode = GAME_MODE_COMPUTER_BLACK; // GAME_MODE_COMPUTER_BLACK GAME_MODE_COMPUTER_WHITE
        }
        DBOut(@"Computer is Black");
        
        // player is white
        [[boardViewController gameController] rotateBoard:NO]; //set board to non rotated position
        
        [boardViewController setPlayerIsWhite:YES];

    }
    else
    {
        // player is black
        if([Options sharedOptions].autoMoveOpponent) // if auto move is on change game mode
        {
            [Options sharedOptions].gameMode = GAME_MODE_COMPUTER_WHITE; // GAME_MODE_COMPUTER_BLACK GAME_MODE_COMPUTER_WHITE
        }
        DBOut(@"Computer is White");
        
        // player is black
        [[boardViewController gameController] rotateBoard:YES]; //set board to rotated position
        
        [boardViewController setPlayerIsWhite:NO];
    }
    
    // how much time to give the computer -- MAKE THIS AN OPTION
    // no longer needed made an option and is loaded from userdefaults - see Options.m
    //[Options sharedOptions].gameLevel = gl_saved;//LEVEL_10S_PER_MOVE;
    
    // setup the game
    [ [boardViewController gameController]  gameFromFEN: [chessModel getFEN]]; //@"4k3/8/3Q4/2B5/8/8/1K6/8 w - - 0 1"];
    DBOut(@"FEN: %@",[chessModel getFEN]);
    
    // change title of back button
    //self.navigationItem.backBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Screenshot" style:UIBarButtonItemStyleBordered target:nil action:nil] autorelease];
    // SEG v1.3 -- removed this as button is added now later
//    self.navigationItem.backBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Screenshot" style:UIBarButtonItemStylePlain target:nil action:nil] autorelease];

    
    
    [self.navigationController pushViewController:boardViewController animated:YES];
    
    [[boardViewController gameController] startEngine];
    
    // if it is not the players turn at the begining -  && auto-move opponent is on, make first move
    if(!chessModel.isPlayersTurnAtBeginning && [Options sharedOptions].autoMoveOpponent)
    {
        [boardViewController moveNow];
    }
    // SEG v1.2-- if it is the players turn and we are requested to autoMakeFirstMove -- trigger the move
    else if(autoMove  &&  chessModel.isPlayersTurnAtBeginning)
    {
       // NSLog(@"Auto MOVE");
        [boardViewController makeFirstMoveAfterSmallDelay];
    }
}


#pragma mark -
#pragma mark MBProgressHUDDelegate methods

- (void)hudWasHidden 
{
    // Remove HUD from screen when the HUD was hidded
    [hud removeFromSuperview];
    [hud release];
	hud = nil;
}


@end
