/*
 ChessCheats, a chess program for the Apple iPhone.
 Copyright (C) 2011 Shawn E. Gano
 
 ChessCheats is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 ChessCheats is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//
//  PieceChangeContentViewController.m
//
//

#import "PieceChangeContentViewController.h"


@implementation PieceChangeContentViewController


@synthesize delegate;


#pragma mark -
#pragma mark Initialization

- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    if ((self = [super initWithStyle:style])) {
        
        // Set the size of the content for 4 rows of data
		self.contentSizeForViewInPopover = CGSizeMake(225, 4 * 44 - 1); // CGSizeMake(100, 4 * 44 - 1);
    }
    return self;
}

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];

	self.view.backgroundColor = [UIColor clearColor];
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}


#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return 14;
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
        
    
    if([indexPath row] == 0 )
    {
        cell.textLabel.text = @"None";
        cell.imageView.image = nil; // no image
    }
    else if([indexPath row] == 1 )
    {
        cell.textLabel.text = @"EP: En passant";
        cell.imageView.image = nil;
    }
    else if([indexPath row] == 2 )
    {
        cell.textLabel.text = @"White Pawn (P)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaWPawn"];
    }
    else if([indexPath row] == 3 )
    {
        cell.textLabel.text = @"Black Pawn (p)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaBPawn"];
    }
    else if([indexPath row] == 4 )
    {
        cell.textLabel.text = @"White Rook (R)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaWRook"];
    }
    else if([indexPath row] == 5 )
    {
        cell.textLabel.text = @"Black Rook (r)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaBRook"];
    }
    else if([indexPath row] == 6 )
    {
        cell.textLabel.text = @"White Knight (N)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaWKnight"];
    }
    else if([indexPath row] == 7 )
    {
        cell.textLabel.text = @"Black Knight (n)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaBKnight"];
    }
    else if([indexPath row] == 8 )
    {
        cell.textLabel.text = @"White Bishop (B)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaWBishop"];
    }
    else if([indexPath row] == 9 )
    {
        cell.textLabel.text = @"Black Bishop (b)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaBBishop"];
    }
    else if([indexPath row] == 10 )
    {
        cell.textLabel.text = @"White Queen (Q)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaWQueen"];
    }
    else if([indexPath row] == 11 )
    {
        cell.textLabel.text = @"Black Queen (q)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaBQueen"];
    }
    else if([indexPath row] == 12 )
    {
        cell.textLabel.text = @"White King (K)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaWKing"];
    }
    else if([indexPath row] == 13 )
    {
        cell.textLabel.text = @"Black King (k)";
        cell.imageView.image = [UIImage imageNamed:@"AlphaBKing"];
    }
    else
    {
        // Configure the cell...
        cell.textLabel.text = [NSString stringWithFormat:@"Item %d", [indexPath row]]; 
        cell.imageView.image = nil; // no image
    }
    
    
	cell.textLabel.textColor = [UIColor whiteColor];
    return cell;
}

-(void)scrollToPiece:(NSString *)pieceStr
{
    // scroll to piece
    
    //[[self tableView] scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:9 inSection:0] atScrollPosition:UITableViewScrollPositionMiddle animated:NO];
    
    int row = 0;  // none
    
    if([pieceStr length] > 1)
    {
        row = 1; // en passant
    }
    else if([pieceStr compare:@"P"]==NSOrderedSame) 
    {
        row = 2;
    }
    else if([pieceStr compare:@"p"]==NSOrderedSame) 
    {
        row = 3;
    }
    else if([pieceStr compare:@"R"]==NSOrderedSame) 
    {
        row = 4;
    }
    else if([pieceStr compare:@"r"]==NSOrderedSame) 
    {
        row = 5;
    }
    else if([pieceStr compare:@"N"]==NSOrderedSame) 
    {
        row = 6;
    }
    else if([pieceStr compare:@"n"]==NSOrderedSame) 
    {
        row = 7;
    }
    else if([pieceStr compare:@"B"]==NSOrderedSame) 
    {
        row = 8;
    }
    else if([pieceStr compare:@"b"]==NSOrderedSame) 
    {
        row = 9;
    }
    else if([pieceStr compare:@"Q"]==NSOrderedSame) 
    {
        row = 10;
    }
    else if([pieceStr compare:@"q"]==NSOrderedSame) 
    {
        row = 11;
    }
    else if([pieceStr compare:@"K"]==NSOrderedSame) 
    {
        row = 12;
    }
    else if([pieceStr compare:@"k"]==NSOrderedSame) 
    {
        row = 13;
    }
    
    [[self tableView] selectRowAtIndexPath:[NSIndexPath indexPathForRow:row inSection:0] animated:YES scrollPosition:UITableViewScrollPositionMiddle];
}


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
   
    // For the first two items (0=none, 1 = en passant) -- deal with them specially
    if(indexPath.row == 0)
    {
        [self.delegate pieceChangeSelectionString:@""];
    }
    else if(indexPath.row == 1)
    {
        [self.delegate pieceChangeSelectionString:@"EP"];
    }
    else
    {
        // For all the pieces -- the value is the second to last letter in the text label
        UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
        //DBOut ( @"You selected row: %@", [cell.textLabel.text substringWithRange:NSMakeRange ([cell.textLabel.text length]-2, 1)]); 
        NSString * newPieceString = [cell.textLabel.text substringWithRange:NSMakeRange ([cell.textLabel.text length]-2, 1)];
        
        [self.delegate pieceChangeSelectionString:newPieceString];
	}
    
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end

