/*
 ChessCheats, a chess program for the Apple iPhone.
 Copyright (C) 2011 Shawn E. Gano
 
 ChessCheats is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 ChessCheats is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//
//  ScreenshotOverlayView.m
//  ChessCheats
//
//  Created by sgano on 4/12/11.
//  Copyright 2011 Shawn E. Gano. All rights reserved.
//

#import "ScreenshotOverlayView.h"



@implementation ScreenshotOverlayView

@synthesize chessModel,screenshotImageView;
@synthesize popoverController;


- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) 
    {
        // Initialization code
    }
    return self;
}

// when loaded from xib
- (id)initWithCoder:(NSCoder *)coder {
    self = [super initWithCoder:coder];
    if (self) 
    {

    }
    return self;
}


-(void)updateDisplay // different instances get called here after memory warning between reprocessing and touch events
{
    [self setNeedsDisplay];
}


-(void)clearBoard
{
    // this clears the board (and the model data) -- should be used before processing a new image
    
    // clear last picked
    lastX = -1;
    lastY = -1;
    lastXBoardCoord = -1;
    lastYBoardCoord = -1;
    
    // set flag such that image hasn't been processed -- makes the drawing of the items on the board stop
    [chessModel setHasImageBeenProcessed:NO]; // turns off
    
    [self setNeedsDisplay]; // repaint
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{
    //DBOut(@"----------- DRAWING =======================");
    
    // Drawing code
    
    if([chessModel getScreenShotImage] == nil || screenshotImageView == nil)
    {
        return; // nothing to really do
    }
    
    float imageH = [chessModel getScreenShotImage].size.height;
    float imageW = [chessModel getScreenShotImage].size.width;
    
    float viewH = screenshotImageView.bounds.size.height;
    float viewW = screenshotImageView.bounds.size.width;
    
    // Ascpect Ratios
    float imageAR = imageH/imageW;
    float viewAR = viewH/viewW;
    
    // location and size of image
    //DBOut(@" %f, %f, %f, %f",imageH,imageW,viewH,viewW);
    
    // data for screen shot location
    float x0 = 0;
    float y0 = 0;
    float width = 0;
    float height = 0;
    
    if(imageAR > viewAR )
    {
        // height bound
        height = viewH;
        width = height/imageAR;
    }
    else
    {
        // width bound
        width = viewW;
        height = width * imageAR;
    }
    
    x0 = (viewW - width)/2.0f;
    y0 = (viewH - height)/2.0f;
    
    //DBOut(@"x0,y0 = %f, %f",x0,y0);
    
    float drawScale = height/imageH; // scale of the image drawn
    
    // chop area of screenshot to be analyzed
    float screenShotScale = [chessModel getScreenShotImage].size.width/320.0f;// in case of retina screenshot
    
    //DBOut(@"Screenshot scale: %f",screenShotScale);
    
    
    
    // get CGContext - like the drawing canvas
    CGContextRef c = UIGraphicsGetCurrentContext();
    
    
    // set properties of the line to draw
    CGContextSetLineWidth(c, 1.0);

    [[UIColor greenColor] set];
    
       
    
    
    /// LOOP THOUGH ALL SQUARES AND PAINT THEM
    if([chessModel hasImageBeenProcessed]) // if image has been processed
    {
        
        
        if([[chessModel getFEN] length] > 1) // fen has been solved for
        {
            UIFont* font;
            if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
            {
                font = [UIFont fontWithName:@"Helvetica" size:18]; // iPad 
            }
            else
            {
                font = [UIFont fontWithName:@"Helvetica" size:12]; // iPhone
            }
            
            for(int i=0;i<8;i++)
            {
                for(int j=0;j<8;j++)
                {
                    // draw it for fun
                    // -- pick color for square based on status (down below)
                    [[UIColor greenColor] set]; // green is default
                    CGContextStrokeRect(c, CGRectMake(x0+i*sizeOfSquare*drawScale*screenShotScale, y0+percentageDownStartOfBoard*imageH*drawScale+j*sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale));
                    
                    // ID each square's contents
                                       // draw string on board
                    [[UIColor blueColor] set]; // blueColor
                    [[chessModel peiceOnBoard_file:i rank:j] drawAtPoint:CGPointMake(x0+i*sizeOfSquare*drawScale*screenShotScale, y0+percentageDownStartOfBoard*imageH*drawScale+j*sizeOfSquare*drawScale*screenShotScale) withFont:font];
                } // j
            }// i
            
            // LOOP AGAIN TO DRAW BOXES WITH STATUS OTHER THAN GREEN!
            // -- pick color for square based on status
            CGContextSetLineWidth(c, 2.0);
            for(int i=0;i<8;i++)
            {
                for(int j=0;j<8;j++)
                {
                    // draw it for fun
                    // -- pick color for square based on status
                    if([chessModel errorOnBoard_file:i rank:j] < 0)
                    {
                        [[UIColor brownColor] set]; // set squares
                        CGContextStrokeRect(c, CGRectMake(x0+i*sizeOfSquare*drawScale*screenShotScale, y0+percentageDownStartOfBoard*imageH*drawScale+j*sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale));
                    }
                    // NEEDS ERROR DATA TOLERANCE BEFORE USE
                    else if([chessModel errorOnBoard_file:i rank:j] > 1.0)// 
                    {
                        [[UIColor colorWithRed:1.0 green:105.0/255.0 blue:180.0/255.0 alpha:1.0f] set]; // hot pink - possible error
                        CGContextStrokeRect(c, CGRectMake(x0+i*sizeOfSquare*drawScale*screenShotScale, y0+percentageDownStartOfBoard*imageH*drawScale+j*sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale));
                    }
                }
            }
            CGContextSetLineWidth(c, 1.0); // return to normal
            
            // Debug
            //[chessModel printErrorBoard];
            
        }// fen solved
    } // if image has been processed - around drawn board and peices IDed
    
    
    // text translation
    if(chessModel.hasImageBeenProcessed) // fen has been solved for
    {
        // Who't turn is it boarder and results
        CGRect turn = CGRectMake(x0+whosTurnPercentRight*drawScale*imageW, y0+whosTurnPercentDown*imageH*drawScale, 
                                 whosTurnWidth*drawScale*imageW, y0+(whosTurnHeight)*imageH*drawScale);
        [[UIColor blueColor] set];
        CGContextStrokeRect(c, turn);
        
        UIFont* font;
        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
        {
            font = [UIFont fontWithName:@"Helvetica" size:18]; // iPad 
        }
        else
        {
            font = [UIFont fontWithName:@"Helvetica" size:11]; // iPhone
        }
        
        int yShift = 13;
        // iPad needs a little more shift
        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
        {
            yShift = 20;
        }
        
        
        if(chessModel.isPlayersTurnAtBeginning)
        {
            // Your
            [@"Your" drawAtPoint:CGPointMake(x0+whosTurnPercentRight*drawScale*imageW, y0+(whosTurnPercentDown)*imageH*drawScale-yShift) withFont:font];
        }
        else
        {
            // Their
            [@"Their" drawAtPoint:CGPointMake(x0+whosTurnPercentRight*drawScale*imageW, y0+(whosTurnPercentDown)*imageH*drawScale-yShift) withFont:font];
        }
        
        /////////////////////////////////////
        // Draw Player's Color area
        /////////////////////////////////////
        CGRect playerColorRect = CGRectMake(x0+playerColorPercentRight*drawScale*imageW, y0+playerColorPercentDown*imageH*drawScale, 
                                 playerColorWidth*drawScale*imageW, y0+(playerColorHeight)*imageH*drawScale);
        
        // draw border
        float borderWidth = 2;
        [[UIColor grayColor] set];
        CGContextFillRect(c, CGRectMake(x0+playerColorPercentRight*drawScale*imageW-borderWidth, y0+playerColorPercentDown*imageH*drawScale-borderWidth, 
                                        playerColorWidth*drawScale*imageW+borderWidth*2, y0+(playerColorHeight)*imageH*drawScale+borderWidth*2));
        
        // BG color
        chessModel.playerWhite ? [[UIColor whiteColor] set]: [[UIColor blackColor] set];
        
        // draw box
        CGContextFillRect(c, playerColorRect);
        
        // SEG v1.1 -- if there is a player color estimate warning draw pink box around outside
        if(chessModel.playerColorEstimateWarning)
        {
            CGContextSetLineWidth(c, 2.0);
            
            [[UIColor colorWithRed:1.0 green:105.0/255.0 blue:180.0/255.0 alpha:1.0f] set]; // hot pink - possible error
            CGContextStrokeRect(c,playerColorRect);
            
            CGContextSetLineWidth(c, 1.0);
        }
        
        // text color - inverse
        (!chessModel.playerWhite) ? [[UIColor whiteColor] set]: [[UIColor blackColor] set];
        
        int xColorShift = 10;
        int yColorShift = 4;
        
        // if iPad need an even bigger font - and moved down and right a little
        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        {
            font = [UIFont fontWithName:@"Helvetica" size:22]; // iPad 
            
            xColorShift = 50;
            yColorShift = 15;
        }
        
        // Your Color:
        [ [NSString stringWithFormat:@"Your Color: %@",chessModel.playerWhite ? @"White" : @"Black"] 
             drawAtPoint:CGPointMake(x0+playerColorPercentRight*drawScale*imageW + xColorShift, y0+(playerColorPercentDown)*imageH*drawScale+yColorShift) withFont:font];
               
    } // image processed

    
    
    
    CGRect rrr = CGRectMake(x0, y0+percentageDownStartOfBoard*imageH*drawScale, width, sizeOfSquare*drawScale*8.0f*screenShotScale);
    [[UIColor greenColor] set];
    // draw box around board
    //CGContextStrokeRect(c, rrr);
    
    // draw last square touched
    // see if last touch was on the actual board
    if( CGRectContainsPoint(rrr, CGPointMake(lastX, lastY)) )
    {
        
        // determine which squre on the board was hit
        int i = (int) ( lastX / (sizeOfSquare*drawScale*screenShotScale) ) - 1;
        
        int j = (int) ( (lastY- percentageDownStartOfBoard*imageH*drawScale)/(sizeOfSquare*drawScale*screenShotScale)  );
        
        // bound checks
        if(i > 7) i=7;
        if(j > 7) j = 7; 
        
        
               // draw red square where picked
        [[UIColor redColor] set];
        CGContextSetLineWidth(c, 2.0);
        CGContextStrokeRect(c, CGRectMake(x0+i*sizeOfSquare*drawScale*screenShotScale, y0+percentageDownStartOfBoard*imageH*drawScale+j*sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale, sizeOfSquare*drawScale*screenShotScale));
        
       
        
    }
    
    
    // Draw En Passant Square
    if([chessModel getEnPassantScreenXCoord] >=0 && [chessModel getEnPassantScreenYCoord] >= 0)
    {
        UIFont* font;
        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) 
        {
            font = [UIFont fontWithName:@"Helvetica" size:18]; // iPad 
        }
        else
        {
            font = [UIFont fontWithName:@"Helvetica" size:12]; // iPhone
        }
        
        
        [[UIColor purpleColor] set];
        int i = [chessModel getEnPassantScreenXCoord];
        int j = [chessModel getEnPassantScreenYCoord];
        [@"EP" drawAtPoint:CGPointMake(x0+i*sizeOfSquare*drawScale*screenShotScale, y0+percentageDownStartOfBoard*imageH*drawScale+j*sizeOfSquare*drawScale*screenShotScale) withFont:font];

    }
    
    
} // draw rect

-(void)closePopups
{
    if (self.popoverController) 
    {
		[self.popoverController dismissPopoverAnimated:YES];
		self.popoverController = nil;
		//currentPopoverCellIndex = -1;
	} 
}


//// Touch 

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    
    //DBOut(@"touches ended");
    UITouch *touch = [touches anyObject];
    
    CGPoint loc = [touch locationInView:self];
    //DBOut(@"loc = %f, %f", loc.x, loc.y);
    
    lastX = loc.x;
    lastY = loc.y;
    
     
    /// show popup!!
    [self closePopups]; // first close any open popups
    
    /////
    // data for screen shot location
    float imageH = [chessModel getScreenShotImage].size.height;
    float imageW = [chessModel getScreenShotImage].size.width;
    
    float viewH = screenshotImageView.bounds.size.height;
    float viewW = screenshotImageView.bounds.size.width;
    
    // Ascpect Ratios
    float imageAR = imageH/imageW;
    float viewAR = viewH/viewW;
    float x0 = 0;
    float y0 = 0;
    float width = 0;
    float height = 0;
    
    if(imageAR > viewAR )
    {
        // height bound
        height = viewH;
        width = height/imageAR;
    }
    else
    {
        // width bound
        width = viewW;
        height = width * imageAR;
    }
    
    x0 = (viewW - width)/2.0f;
    y0 = (viewH - height)/2.0f;
    
    float drawScale = height/imageH; // scale of the image drawn
    float screenShotScale = [chessModel getScreenShotImage].size.width/320.0f;// in case of retina screenshot
    
    CGRect rrr = CGRectMake(x0, y0+percentageDownStartOfBoard*imageH*drawScale, width, sizeOfSquare*drawScale*8.0f*screenShotScale);
    
    
    // see if last touch was on the actual board
    if( CGRectContainsPoint(rrr, CGPointMake(loc.x, loc.y)) )
    {
        //////
        CGRect bounds = CGRectMake(loc.x, loc.y, 1.0, 1.0);//[self rectFromOrigin:loc inset:3.0]; // TILE_INSET
        
        PieceChangeContentViewController *contentViewController = [[PieceChangeContentViewController alloc] initWithStyle:UITableViewStylePlain];
        contentViewController.delegate = self;
        
        //CGRect frame = [tableView cellForRowAtIndexPath:indexPath].frame;
        
        self.popoverController = [[[WEPopoverController alloc] initWithContentViewController:contentViewController] autorelease];
        
        [self.popoverController presentPopoverFromRect:bounds 
                                                inView:self
                              permittedArrowDirections:UIPopoverArrowDirectionDown|UIPopoverArrowDirectionUp
                                              animated:YES];
        
        //currentPopoverCellIndex = indexPath.row;
        
        [contentViewController release];
        
        
        // Save last square selected coordinates
        lastXBoardCoord = (int) ( lastX / (sizeOfSquare*drawScale*screenShotScale) ) - 1;
        
        lastYBoardCoord = (int) ( (lastY- percentageDownStartOfBoard*imageH*drawScale)/(sizeOfSquare*drawScale*screenShotScale)  );
        
        // bound checks
        if(lastXBoardCoord > 7) lastXBoardCoord=7;
        if(lastYBoardCoord > 7) lastYBoardCoord = 7;
        
        
        // select a default option and scroll to it
        if([chessModel getEnPassantScreenXCoord] == lastXBoardCoord && [chessModel getEnPassantScreenYCoord] == lastYBoardCoord)
        {
            [contentViewController scrollToPiece:@"EP"];
        }
        else
        {
            [contentViewController scrollToPiece:[chessModel peiceOnBoard_file:lastXBoardCoord rank:lastYBoardCoord]];
        }
        /////////////////////////////////////
        // DEBUG --- compute white/black percentage
        BOOL debug = NO;
        if(debug)
        {
            [chessModel printBWcomponentsOfSquare_file:lastXBoardCoord rank:lastYBoardCoord];
            
        } // debug
        /////////////////////////////////////
    } // if touch was on actual board
    else if(  CGRectContainsPoint(CGRectMake(x0+whosTurnPercentRight*drawScale*imageW, y0+whosTurnPercentDown*imageH*drawScale, whosTurnWidth*drawScale*imageW, y0+(whosTurnHeight)*imageH*drawScale), CGPointMake(loc.x, loc.y))  ) // if touch in turn
    {
        CGRect bounds = CGRectMake(loc.x, loc.y, 1.0, 1.0);//[self rectFromOrigin:loc inset:3.0]; // TILE_INSET
        
        MoveChangeTableViewController *contentViewController = [[MoveChangeTableViewController alloc] initWithStyle:UITableViewStylePlain];
        contentViewController.delegate = self;
        
        //CGRect frame = [tableView cellForRowAtIndexPath:indexPath].frame;
        
        self.popoverController = [[[WEPopoverController alloc] initWithContentViewController:contentViewController] autorelease];
        
        [self.popoverController presentPopoverFromRect:bounds 
                                                inView:self
                              permittedArrowDirections:UIPopoverArrowDirectionDown|UIPopoverArrowDirectionUp
                                              animated:YES];
        
        //currentPopoverCellIndex = indexPath.row;
        
        [contentViewController scrollToYourTurn:chessModel.isPlayersTurnAtBeginning];
        
        [contentViewController release];
    } // if touch in turn
    // players color
    else if( CGRectContainsPoint( CGRectMake(x0+playerColorPercentRight*drawScale*imageW, y0+playerColorPercentDown*imageH*drawScale, 
                         playerColorWidth*drawScale*imageW, y0+(playerColorHeight)*imageH*drawScale)  , CGPointMake(loc.x, loc.y)) ) // 
    {
        CGRect bounds = CGRectMake(loc.x, loc.y, 1.0, 1.0);//[self rectFromOrigin:loc inset:3.0]; // TILE_INSET
        
        ColorChangeTableViewController *contentViewController = [[ColorChangeTableViewController alloc] initWithStyle:UITableViewStylePlain];
        contentViewController.delegate = self;
        
        //CGRect frame = [tableView cellForRowAtIndexPath:indexPath].frame;
        
        self.popoverController = [[[WEPopoverController alloc] initWithContentViewController:contentViewController] autorelease];
        
        [self.popoverController presentPopoverFromRect:bounds 
                                                inView:self
                              permittedArrowDirections:UIPopoverArrowDirectionDown|UIPopoverArrowDirectionUp
                                              animated:YES];
        
        //currentPopoverCellIndex = indexPath.row;
        
        [contentViewController scrollToColorIsWhite:chessModel.playerWhite]; 
        
        [contentViewController release];

    } // touch in players color 
    
         
    [self updateDisplay]; // redraw
}
////

// Peice Change delegate //////

-(void)pieceChangeSelectionString:(NSString *)string 
{
    
    // close popup
    [self closePopups];
    
    // which square is changing = lastXBoardCoord,lastYBoardCoord
    if([string length] < 2) // anything but EP [piece or blank]
    {
        [chessModel updateXPosition:lastXBoardCoord yPosition:lastYBoardCoord piece:string];
    }
    else
    {
        // HANDLE EN PASSANT
        [chessModel updateXPosition:lastXBoardCoord yPosition:lastYBoardCoord piece:@"EP"];
    }
    
    // redraw
    [self setNeedsDisplay];
}

////////////

-(void)moveChangeSelectionIsYour:(BOOL)isYourTurn
{
    // close popup
    [self closePopups];
    
    // update who's move it is in the chess model
    chessModel.isPlayersTurnAtBeginning = isYourTurn;
    
    // redraw
    [self setNeedsDisplay];
}
/////////////

-(void)colorChangeSelectionPlayerIsWhite:(BOOL)playerIsWhite
{
    // close popup
    [self closePopups];
    
    // update who's move it is in the chess model
    chessModel.playerWhite = playerIsWhite;
    
    // redraw
    [self setNeedsDisplay];
}


/////// popup methods -----------------------
- (void)popoverControllerDidDismissPopover:(WEPopoverController *)popoverController {
    
    DBOut(@"Did dismiss");
}

- (BOOL)popoverControllerShouldDismissPopover:(WEPopoverController *)popoverController {
    DBOut(@"Should dismiss");
    return YES;
}

/////////////////////////////////////////////


- (void)dealloc
{
    [super dealloc];
}

@end
